@@ -23,6 +23,7 @@
  */
 
 #include "gnunet_chat_discourse.h"
+
 #include <gnunet/gnunet_common.h>
 #include <gnunet/gnunet_scheduler_lib.h>
 #include <gnunet/gnunet_time_lib.h>
@@ -33,7 +34,7 @@
 
 struct GNUNET_CHAT_Discourse*
 discourse_create (struct GNUNET_CHAT_Context *context,
-                  const struct GNUNET_ShortHashCode *id)
+                  const struct GNUNET_CHAT_DiscourseId *id)
 {
   GNUNET_assert((context) && (id));
 
@@ -41,7 +42,7 @@ discourse_create (struct GNUNET_CHAT_Context *context,
 
   discourse->context = context;
 
-  GNUNET_memcpy(&(discourse->id), id, sizeof (struct GNUNET_ShortHashCode));
+  GNUNET_memcpy(&(discourse->id), id, sizeof(struct GNUNET_CHAT_DiscourseId));
 
   if (0 != pipe(discourse->pipe))
   {
