@@ -25,13 +25,13 @@
 #ifndef GNUNET_CHAT_DISCOURSE_H_
 #define GNUNET_CHAT_DISCOURSE_H_
 
+#include "gnunet_chat_util.h"
+
 #include <gnunet/gnunet_common.h>
 #include <gnunet/gnunet_messenger_service.h>
 #include <gnunet/gnunet_time_lib.h>
 #include <gnunet/gnunet_util_lib.h>
 
-#include "gnunet_chat_util.h"
-
 struct GNUNET_CHAT_Contact;
 struct GNUNET_CHAT_Context;
 
@@ -53,7 +53,7 @@ struct GNUNET_CHAT_Discourse
 {
   struct GNUNET_CHAT_Context *context;
 
-  struct GNUNET_ShortHashCode id;
+  struct GNUNET_CHAT_DiscourseId id;
   int pipe [2];
 
   struct GNUNET_CHAT_DiscourseSubscription *head;
@@ -74,7 +74,7 @@ struct GNUNET_CHAT_Discourse
  */
 struct GNUNET_CHAT_Discourse*
 discourse_create (struct GNUNET_CHAT_Context *context,
-                  const struct GNUNET_ShortHashCode *id);
+                  const struct GNUNET_CHAT_DiscourseId *id);
 
 /**
  * Destroys a chat <i>discourse</i> and frees its memory.
