@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2021--2024 GNUnet e.V.
+   Copyright (C) 2021--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -29,8 +29,10 @@
 #include "gnunet_chat_message.h"
 #include <gnunet/gnunet_arm_service.h>
 #include <gnunet/gnunet_common.h>
+#include <gnunet/gnunet_messenger_service.h>
 #include <gnunet/gnunet_reclaim_service.h>
 #include <gnunet/gnunet_scheduler_lib.h>
+#include <gnunet/gnunet_util_lib.h>
 
 static const unsigned int initial_map_size_of_handle = 8;
 static const unsigned int minimum_amount_of_other_members_in_group = 2;
@@ -93,6 +95,7 @@ handle_create_from_config (const struct GNUNET_CONFIGURATION_Handle* cfg,
   handle->accounts_tail = NULL;
 
   handle->refreshing = GNUNET_NO;
+  handle->own_contact = NULL;
 
   handle->next = NULL;
   handle->current = NULL;
@@ -116,6 +119,7 @@ handle_create_from_config (const struct GNUNET_CONFIGURATION_Handle* cfg,
   handle->contexts = NULL;
   handle->contacts = NULL;
   handle->groups = NULL;
+  handle->invitations = NULL;
 
   handle->arm = GNUNET_ARM_connect(
     handle->cfg,
@@ -164,6 +168,7 @@ handle_update_key (struct GNUNET_CHAT_Handle *handle)
     GNUNET_free(handle->public_key);
 
   handle->public_key = NULL;
+  handle->own_contact = NULL;
 
   if (!(handle->messenger))
     return;
@@ -331,9 +336,10 @@ handle_connect (struct GNUNET_CHAT_Handle *handle,
   GNUNET_assert(
     (handle) && (account) &&
 		(!(handle->current)) &&
-		(!(handle->groups)) &&
-		(!(handle->contacts)) &&
 		(!(handle->contexts)) &&
+    (!(handle->contacts)) &&
+    (!(handle->groups)) &&
+    (!(handle->invitations)) &&
 		(handle->files)
   );
 
@@ -349,6 +355,8 @@ handle_connect (struct GNUNET_CHAT_Handle *handle,
     initial_map_size_of_handle, GNUNET_NO);
   handle->groups = GNUNET_CONTAINER_multihashmap_create(
     initial_map_size_of_handle, GNUNET_NO);
+  handle->invitations = GNUNET_CONTAINER_multihashmap_create(
+    initial_map_size_of_handle, GNUNET_NO);
 
   handle->gns = GNUNET_GNS_connect(handle->cfg);
 
@@ -389,6 +397,8 @@ handle_disconnect (struct GNUNET_CHAT_Handle *handle)
     GNUNET_YES
   );
 
+  handle->own_contact = NULL;
+
   while (handle->attributes_head)
     internal_attributes_destroy(handle->attributes_head);
 
@@ -480,6 +490,9 @@ handle_disconnect (struct GNUNET_CHAT_Handle *handle)
     GNUNET_free(lobbies);
   }
 
+  handle->own_contact = NULL;
+
+  GNUNET_CONTAINER_multihashmap_destroy(handle->invitations);
   GNUNET_CONTAINER_multihashmap_destroy(handle->groups);
   GNUNET_CONTAINER_multishortmap_destroy(handle->contacts);
   GNUNET_CONTAINER_multihashmap_destroy(handle->contexts);
@@ -488,6 +501,7 @@ handle_disconnect (struct GNUNET_CHAT_Handle *handle)
   handle->contexts = NULL;
   handle->contacts = NULL;
   handle->groups = NULL;
+  handle->invitations = NULL;
 
   if (handle->connection)
     GNUNET_SCHEDULER_cancel(handle->connection);
@@ -1089,12 +1103,12 @@ handle_process_records (struct GNUNET_CHAT_Handle *handle,
   if (!record)
     return NULL;
 
-  struct GNUNET_HashCode key;
-  GNUNET_memcpy (&key, &(record->key), sizeof(key));
+  union GNUNET_MESSENGER_RoomKey key;
+  GNUNET_memcpy (&(key.hash), &(record->key), sizeof(key));
 
   struct GNUNET_CHAT_Context *context = GNUNET_CONTAINER_multihashmap_get(
     handle->contexts,
-    &key
+    &(key.hash)
   );
 
   if ((context) && (context->room))
@@ -1121,10 +1135,8 @@ handle_process_records (struct GNUNET_CHAT_Handle *handle,
   context = context_create_from_room(handle, room);
   context_read_records(context, label, count, data);
 
-  handle_send_room_name(handle, room);
-
   if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
-      handle->contexts, &key, context,
+      handle->contexts, &(key.hash), context,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
   {
     context_destroy(context);
@@ -1141,7 +1153,7 @@ handle_process_records (struct GNUNET_CHAT_Handle *handle,
     group_publish(group);
 
   if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
-      handle->groups, &key, group,
+      handle->groups, &(key.hash), group,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
     group_destroy(group);
 
