@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2021--2024 GNUnet e.V.
+   Copyright (C) 2021--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -719,6 +719,19 @@ on_handle_internal_message_callback(void *cls)
   handle->msg_cb(handle->msg_cls, context, internal->msg);
 }
 
+static enum GNUNET_GenericReturnValue
+it_invitation_update (GNUNET_UNUSED void *cls,
+                      GNUNET_UNUSED const struct GNUNET_HashCode *key,
+                      void *value)
+{
+  struct GNUNET_CHAT_Invitation *invitation = (struct GNUNET_CHAT_Invitation*) value;
+
+  if (invitation)
+    invitation_update(invitation);
+
+  return GNUNET_YES;
+}
+
 void
 on_handle_message_callback(void *cls)
 {
@@ -742,7 +755,7 @@ on_handle_message_callback(void *cls)
   struct GNUNET_TIME_Relative task_delay;
   switch (message->msg->header.kind)
   {
-    case GNUNET_MESSENGER_KIND_DELETE:
+    case GNUNET_MESSENGER_KIND_DELETION:
     {
       const struct GNUNET_TIME_Relative delay = GNUNET_TIME_relative_ntoh(
 	      message->msg->body.deletion.delay
@@ -796,6 +809,10 @@ on_handle_message_callback(void *cls)
         context->invites, &(message->hash), invitation,
         GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
 	      invitation_destroy(invitation);
+      else
+        GNUNET_CONTAINER_multihashmap_put(
+          handle->invitations, &(invitation->key.hash), invitation,
+          GNUNET_CONTAINER_MULTIHASHMAPOPTION_MULTIPLE);
       break;
     }
     case GNUNET_MESSENGER_KIND_FILE:
@@ -880,6 +897,24 @@ skip_msg_handing:
       contact_update_join(contact, context, 
         &(message->hash), message->flags);
       
+      GNUNET_CONTAINER_multihashmap_get_multiple(
+        handle->invitations,
+        GNUNET_MESSENGER_room_get_key(context->room),
+        it_invitation_update, handle);
+      
+      if ((GNUNET_MESSENGER_FLAG_SENT & message->flags) &&
+          (GNUNET_MESSENGER_FLAG_RECENT & message->flags))
+        handle_send_room_name(handle, context->room);
+      
+      break;
+    }
+    case GNUNET_MESSENGER_KIND_LEAVE:
+    {
+      GNUNET_CONTAINER_multihashmap_get_multiple(
+        handle->invitations,
+        GNUNET_MESSENGER_room_get_key(context->room),
+        it_invitation_update, handle);
+      
       break;
     }
     case GNUNET_MESSENGER_KIND_KEY:
@@ -927,9 +962,9 @@ skip_msg_handing:
       );
       break;
     }
-    case GNUNET_MESSENGER_KIND_SUBSCRIBE:
+    case GNUNET_MESSENGER_KIND_SUBSCRIBTION:
     {
-      const struct GNUNET_ShortHashCode *sid = &(message->msg->body.subscribe.discourse);
+      const struct GNUNET_ShortHashCode *sid = &(message->msg->body.subscription.discourse);
       struct GNUNET_CHAT_Discourse *discourse = GNUNET_CONTAINER_multishortmap_get(
         context->discourses, sid
       );
@@ -949,24 +984,24 @@ skip_msg_handing:
         }
       }
 
-      enum GNUNET_GenericReturnValue subscribtion_update = GNUNET_NO;
+      enum GNUNET_GenericReturnValue subscription_update = GNUNET_NO;
 
-      if (GNUNET_MESSENGER_FLAG_SUBSCRIPTION_UNSUBSCRIBE & message->msg->body.subscribe.flags)
+      if (GNUNET_MESSENGER_FLAG_SUBSCRIPTION_UNSUBSCRIBE & message->msg->body.subscription.flags)
         discourse_unsubscribe(
           discourse,
           contact,
           GNUNET_TIME_absolute_ntoh(message->msg->header.timestamp),
-          GNUNET_TIME_relative_ntoh(message->msg->body.subscribe.time)
+          GNUNET_TIME_relative_ntoh(message->msg->body.subscription.time)
         );
       else
-        subscribtion_update = discourse_subscribe(
+        subscription_update = discourse_subscribe(
           discourse,
           contact,
           GNUNET_TIME_absolute_ntoh(message->msg->header.timestamp),
-          GNUNET_TIME_relative_ntoh(message->msg->body.subscribe.time)
+          GNUNET_TIME_relative_ntoh(message->msg->body.subscription.time)
         );
       
-      if (GNUNET_YES == subscribtion_update)
+      if (GNUNET_YES == subscription_update)
         message->flags |= GNUNET_MESSENGER_FLAG_UPDATE;
 
       break;
@@ -1016,11 +1051,11 @@ on_handle_message (void *cls,
     handle->contexts, GNUNET_MESSENGER_room_get_key(room)
   );
 
-  context_request_message(context, &(msg->header.previous));
-
   if (GNUNET_MESSENGER_KIND_MERGE == msg->header.kind)
     context_request_message(context, &(msg->body.merge.previous));
 
+  context_request_message(context, &(msg->header.previous));
+
   if ((GNUNET_CHAT_KIND_UNKNOWN == util_message_kind_from_kind(msg->header.kind)) ||
       (GNUNET_OK != intern_provide_contact_for_member(handle, sender, NULL)))
     return;
@@ -1095,7 +1130,7 @@ on_handle_message (void *cls,
 handle_callback:
   switch (msg->header.kind)
   {
-    case GNUNET_MESSENGER_KIND_DELETE:
+    case GNUNET_MESSENGER_KIND_DELETION:
     {
       dependency = &(msg->body.deletion.hash);
       break;
