@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2021--2024 GNUnet e.V.
+   Copyright (C) 2021--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -933,6 +933,18 @@ GNUNET_CHAT_iterate_contacts (struct GNUNET_CHAT_Handle *handle,
 }
 
 
+struct GNUNET_CHAT_Contact*
+GNUNET_CHAT_get_own_contact (struct GNUNET_CHAT_Handle *handle)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!(handle->own_contact))
+    GNUNET_CHAT_iterate_contacts (handle, it_handle_find_own_contact, NULL);
+
+  return handle->own_contact;
+}
+
+
 const char*
 GNUNET_CHAT_account_get_name (const struct GNUNET_CHAT_Account *account)
 {
@@ -1012,7 +1024,7 @@ GNUNET_CHAT_account_get_user_pointer (const struct GNUNET_CHAT_Account *account)
 
 struct GNUNET_CHAT_Group *
 GNUNET_CHAT_group_create (struct GNUNET_CHAT_Handle *handle,
-			                    const char* topic)
+                          const char* topic)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
@@ -1020,14 +1032,16 @@ GNUNET_CHAT_group_create (struct GNUNET_CHAT_Handle *handle,
       (!(handle->groups)) || (!(handle->contexts)))
     return NULL;
 
-  struct GNUNET_HashCode key;
+  union GNUNET_MESSENGER_RoomKey key;
+  GNUNET_MESSENGER_create_room_key(
+    &key,
+    topic,
+    topic? GNUNET_YES : GNUNET_NO,
+    GNUNET_YES,
+    GNUNET_NO
+  );
 
-  if (topic)
-    GNUNET_CRYPTO_hash(topic, strlen(topic), &key);
-  else
-    GNUNET_CRYPTO_random_block(GNUNET_CRYPTO_QUALITY_WEAK, &key, sizeof(key));
-
-  if (GNUNET_YES == GNUNET_CONTAINER_multihashmap_contains(handle->contexts, &key))
+  if (GNUNET_YES == GNUNET_CONTAINER_multihashmap_contains(handle->contexts, &(key.hash)))
     return NULL;
 
   struct GNUNET_MESSENGER_Room *room = GNUNET_MESSENGER_open_room(
@@ -1038,15 +1052,12 @@ GNUNET_CHAT_group_create (struct GNUNET_CHAT_Handle *handle,
     return NULL;
 
   struct GNUNET_CHAT_Context *context = context_create_from_room(handle, room);
-
-  handle_send_room_name(handle, room);
-
   context->type = GNUNET_CHAT_CONTEXT_TYPE_GROUP;
 
   util_set_name_field(topic, &(context->topic));
 
   if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
-      handle->contexts, &key, context,
+      handle->contexts, &(key.hash), context,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
     goto destroy_context;
 
@@ -1056,7 +1067,7 @@ GNUNET_CHAT_group_create (struct GNUNET_CHAT_Handle *handle,
     group_publish(group);
 
   if (GNUNET_OK == GNUNET_CONTAINER_multihashmap_put(
-      handle->groups, &key, group,
+      handle->groups, &(key.hash), group,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
   {
     context_write_records(context);
@@ -1065,7 +1076,7 @@ GNUNET_CHAT_group_create (struct GNUNET_CHAT_Handle *handle,
 
   group_destroy(group);
 
-  GNUNET_CONTAINER_multihashmap_remove(handle->contexts, &key, context);
+  GNUNET_CONTAINER_multihashmap_remove(handle->contexts, &(key.hash), context);
 
 destroy_context:
   context_destroy(context);
@@ -1456,12 +1467,15 @@ GNUNET_CHAT_group_invite_contact (struct GNUNET_CHAT_Group *group,
   if (!context)
     return GNUNET_SYSERR;
 
-  const struct GNUNET_HashCode *key = GNUNET_MESSENGER_room_get_key(
-    group->context->room
+  union GNUNET_MESSENGER_RoomKey key;
+  GNUNET_memcpy(
+    &(key.hash),
+    GNUNET_MESSENGER_room_get_key(group->context->room),
+    sizeof(key.hash)
   );
 
   handle_send_room_name(group->handle, GNUNET_MESSENGER_open_room(
-    group->handle->messenger, key
+    group->handle->messenger, &key
   ));
 
   struct GNUNET_MESSENGER_Message msg;
@@ -1469,7 +1483,7 @@ GNUNET_CHAT_group_invite_contact (struct GNUNET_CHAT_Group *group,
 
   msg.header.kind = GNUNET_MESSENGER_KIND_INVITE;
   GNUNET_CRYPTO_get_peer_identity(group->handle->cfg, &(msg.body.invite.door));
-  GNUNET_memcpy(&(msg.body.invite.key), key, sizeof(msg.body.invite.key));
+  GNUNET_memcpy(&(msg.body.invite.key), &key, sizeof(msg.body.invite.key));
 
   GNUNET_MESSENGER_send_message(context->room, &msg, contact->member);
   return GNUNET_OK;
@@ -1551,18 +1565,27 @@ GNUNET_CHAT_group_get_context (struct GNUNET_CHAT_Group *group)
 
 
 enum GNUNET_GenericReturnValue
-GNUNET_CHAT_context_get_status (const struct GNUNET_CHAT_Context *context)
+GNUNET_CHAT_context_get_status (struct GNUNET_CHAT_Context *context)
 {
   GNUNET_CHAT_VERSION_ASSERT();
 
   if ((!context) || (!(context->room)))
     return GNUNET_SYSERR;
 
-  if ((GNUNET_CHAT_CONTEXT_TYPE_UNKNOWN == context->type) ||
-      (1 >= GNUNET_MESSENGER_iterate_members(context->room, NULL, NULL)))
-    return GNUNET_NO;
+  switch (context->type) {
+    case GNUNET_CHAT_CONTEXT_TYPE_CONTACT:
+    {
+      const struct GNUNET_CHAT_Contact *contact = GNUNET_CHAT_context_get_contact(
+        context
+      );
 
-  return GNUNET_OK;
+      return contact? GNUNET_OK : GNUNET_NO;
+    }
+    case GNUNET_CHAT_CONTEXT_TYPE_GROUP:
+      return GNUNET_OK;
+    default:
+      return GNUNET_NO;
+  }
 }
 
 
@@ -1581,62 +1604,84 @@ GNUNET_CHAT_context_request (struct GNUNET_CHAT_Context *context)
   if ((!handle) || (!(context->contact)))
     return GNUNET_SYSERR;
 
-  struct GNUNET_CHAT_Contact *contact = contact_create_from_member(
+  struct GNUNET_CHAT_Contact *contact = handle_get_contact_from_messenger(
     handle, context->contact
   );
 
   if (!contact)
     return GNUNET_SYSERR;
 
+  enum GNUNET_GenericReturnValue owned = GNUNET_CHAT_contact_is_owned(
+    contact
+  );
+
   context->type = GNUNET_CHAT_CONTEXT_TYPE_CONTACT;
 
   struct GNUNET_CHAT_Context *other = contact_find_context(
-    contact,
-    GNUNET_YES
+    contact, GNUNET_YES
   );
 
   if (!other)
-    goto cleanup_contact;
+    return GNUNET_SYSERR;
 
-  struct GNUNET_HashCode key;
-  GNUNET_CRYPTO_random_block(GNUNET_CRYPTO_QUALITY_WEAK, &key, sizeof(key));
-
-  if (GNUNET_YES == GNUNET_CONTAINER_multihashmap_contains(
-      handle->contexts, &key))
-    goto cleanup_contact;
-
-  struct GNUNET_MESSENGER_Room *room = GNUNET_MESSENGER_open_room(
-    handle->messenger, &key
+  union GNUNET_MESSENGER_RoomKey key;
+  GNUNET_MESSENGER_create_room_key(
+    &key,
+    NULL,
+    GNUNET_YES == owned? GNUNET_YES : GNUNET_NO,
+    GNUNET_NO,
+    GNUNET_YES == owned? GNUNET_YES : GNUNET_NO
   );
 
+  if (GNUNET_YES == GNUNET_CONTAINER_multihashmap_contains(
+      handle->contexts, &(key.hash)))
+    return GNUNET_SYSERR;
+
+  struct GNUNET_MESSENGER_Room *room;
+  if (GNUNET_YES == owned)
+  {
+    struct GNUNET_PeerIdentity door;
+    if (GNUNET_OK == GNUNET_CRYPTO_get_peer_identity(
+          handle->cfg, &door))
+      room = GNUNET_MESSENGER_enter_room(
+        handle->messenger,
+        &door,
+        &key
+      );
+    else
+      room = NULL;
+  }
+  else
+    room = GNUNET_MESSENGER_open_room(
+      handle->messenger, &key
+    );
+
   if (!room)
-    goto cleanup_contact;
+    return GNUNET_SYSERR;
 
   context_update_room(context, room, GNUNET_YES);
 
   if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
-      handle->contexts, &key, context,
+      handle->contexts, &(key.hash), context,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
-    goto cleanup_room;
+  {
+    context_update_room(context, NULL, GNUNET_YES);
+    return GNUNET_SYSERR;
+  }
 
-  handle_send_room_name(handle, room);
+  if (GNUNET_YES != owned)
+  {
+    struct GNUNET_MESSENGER_Message msg;
+    memset(&msg, 0, sizeof(msg));
 
-  struct GNUNET_MESSENGER_Message msg;
-  memset(&msg, 0, sizeof(msg));
+    msg.header.kind = GNUNET_MESSENGER_KIND_INVITE;
+    GNUNET_CRYPTO_get_peer_identity(handle->cfg, &(msg.body.invite.door));
+    GNUNET_memcpy(&(msg.body.invite.key), &key, sizeof(msg.body.invite.key));
 
-  msg.header.kind = GNUNET_MESSENGER_KIND_INVITE;
-  GNUNET_CRYPTO_get_peer_identity(handle->cfg, &(msg.body.invite.door));
-  GNUNET_memcpy(&(msg.body.invite.key), &key, sizeof(msg.body.invite.key));
+    GNUNET_MESSENGER_send_message(other->room, &msg, context->contact);
+  }
 
-  GNUNET_MESSENGER_send_message(other->room, &msg, context->contact);
   return GNUNET_OK;
-
-cleanup_room:
-  GNUNET_MESSENGER_close_room(room);
-
-cleanup_contact:
-  contact_destroy(contact);
-  return GNUNET_SYSERR;
 }
 
 
@@ -1653,8 +1698,18 @@ GNUNET_CHAT_context_get_contact (struct GNUNET_CHAT_Context *context)
 
   struct GNUNET_MESSENGER_Room *room = context->room;
   struct GNUNET_CHAT_RoomFindContact find;
+  union GNUNET_MESSENGER_RoomKey key;
 
-  find.ignore_key = GNUNET_MESSENGER_get_key(context->handle->messenger);
+  GNUNET_memcpy(&(key.hash), GNUNET_MESSENGER_room_get_key(room), sizeof(key.hash));
+
+  if (key.code.group_bit)
+    return NULL;
+
+  if (! key.code.feed_bit)
+    find.ignore_key = GNUNET_MESSENGER_get_key(context->handle->messenger);
+  else
+    find.ignore_key = NULL;
+  
   find.contact = NULL;
 
   int member_count = GNUNET_MESSENGER_iterate_members(
@@ -1977,19 +2032,19 @@ GNUNET_CHAT_context_open_discourse (struct GNUNET_CHAT_Context *context,
   struct GNUNET_MESSENGER_Message msg;
   memset(&msg, 0, sizeof(msg));
 
-  msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBE;
+  msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBTION;
   GNUNET_memcpy(
-    &(msg.body.subscribe.discourse),
+    &(msg.body.subscription.discourse),
     &sid,
     sizeof(struct GNUNET_ShortHashCode)
   );
 
-  const struct GNUNET_TIME_Relative subscribtion_time = GNUNET_TIME_relative_multiply(
+  const struct GNUNET_TIME_Relative subscription_time = GNUNET_TIME_relative_multiply(
     GNUNET_TIME_relative_get_second_(), 10
   );
 
-  msg.body.subscribe.time = GNUNET_TIME_relative_hton(subscribtion_time);
-  msg.body.subscribe.flags = GNUNET_MESSENGER_FLAG_SUBSCRIPTION_KEEP_ALIVE;
+  msg.body.subscription.time = GNUNET_TIME_relative_hton(subscription_time);
+  msg.body.subscription.flags = GNUNET_MESSENGER_FLAG_SUBSCRIPTION_KEEP_ALIVE;
 
   GNUNET_MESSENGER_send_message(
     context->room,
@@ -2292,6 +2347,31 @@ GNUNET_CHAT_message_get_text (const struct GNUNET_CHAT_Message *message)
 }
 
 
+void
+GNUNET_CHAT_message_set_user_pointer (struct GNUNET_CHAT_Message *message,
+                                      void *user_pointer)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!message)
+    return;
+
+  message->user_pointer = user_pointer;
+}
+
+
+void*
+GNUNET_CHAT_message_get_user_pointer (const struct GNUNET_CHAT_Message *message)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if (!message)
+    return NULL;
+
+  return message->user_pointer;
+}
+
+
 struct GNUNET_CHAT_Account*
 GNUNET_CHAT_message_get_account (const struct GNUNET_CHAT_Message *message)
 {
@@ -2356,10 +2436,10 @@ GNUNET_CHAT_message_get_discourse (const struct GNUNET_CHAT_Message *message)
   
   struct GNUNET_CHAT_Discourse *discourse;
   
-  if (GNUNET_MESSENGER_KIND_SUBSCRIBE == message->msg->header.kind)
+  if (GNUNET_MESSENGER_KIND_SUBSCRIBTION == message->msg->header.kind)
     discourse = GNUNET_CONTAINER_multishortmap_get(
       message->context->discourses,
-      &(message->msg->body.subscribe.discourse));
+      &(message->msg->body.subscription.discourse));
   else if (GNUNET_MESSENGER_KIND_TALK == message->msg->header.kind)
     discourse = GNUNET_CONTAINER_multishortmap_get(
       message->context->discourses,
@@ -2382,7 +2462,7 @@ GNUNET_CHAT_message_get_target (const struct GNUNET_CHAT_Message *message)
 
   struct GNUNET_CHAT_Message *target;
 
-  if (GNUNET_MESSENGER_KIND_DELETE == message->msg->header.kind)
+  if (GNUNET_MESSENGER_KIND_DELETION == message->msg->header.kind)
     target = GNUNET_CONTAINER_multihashmap_get(
 	message->context->messages, &(message->msg->body.deletion.hash));
   else if (GNUNET_MESSENGER_KIND_TAG == message->msg->header.kind)
@@ -2899,15 +2979,60 @@ GNUNET_CHAT_invitation_accept (struct GNUNET_CHAT_Invitation *invitation)
   if (!invitation)
     return;
 
+  struct GNUNET_CHAT_Handle *handle;
+  handle = invitation->context->handle;
+
+  if (GNUNET_YES == GNUNET_CONTAINER_multihashmap_contains(
+      handle->contexts, &(invitation->key.hash)))
+    return;
+
   struct GNUNET_PeerIdentity door;
   GNUNET_PEER_resolve(invitation->door, &door);
 
-  struct GNUNET_MESSENGER_Room *room = GNUNET_MESSENGER_enter_room(
+  struct GNUNET_MESSENGER_Room *room;
+  room = GNUNET_MESSENGER_enter_room(
     invitation->context->handle->messenger,
     &door, &(invitation->key)
   );
 
-  handle_send_room_name(invitation->context->handle, room);
+  if (!room)
+    return;
+
+  struct GNUNET_CHAT_Context *context;
+  context = context_create_from_room(handle, room);
+
+  if (!context)
+    return;
+
+  if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
+      handle->contexts, &(invitation->key.hash), context,
+      GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
+    goto destroy_context;
+  
+  if (GNUNET_CHAT_CONTEXT_TYPE_GROUP != context->type)
+  {
+    context_write_records(context);
+    return;
+  }
+
+  struct GNUNET_CHAT_Group *group;
+  group = group_create_from_context(handle, context);
+
+  if (GNUNET_OK == GNUNET_CONTAINER_multihashmap_put(
+      handle->groups, &(invitation->key.hash), group,
+      GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
+  {
+    context_write_records(context);
+    return;
+  }
+
+  group_destroy(group);
+
+  GNUNET_CONTAINER_multihashmap_remove(
+    handle->contexts, &(invitation->key.hash), context);
+
+destroy_context:
+  context_destroy(context);
 }
 
 
@@ -2948,7 +3073,7 @@ GNUNET_CHAT_invitation_is_accepted (const struct GNUNET_CHAT_Invitation *invitat
 
   return GNUNET_CONTAINER_multihashmap_contains(
     invitation->context->handle->contexts,
-    &(invitation->key)
+    &(invitation->key.hash)
   );
 }
 
@@ -2974,6 +3099,20 @@ GNUNET_CHAT_invitation_is_rejected (const struct GNUNET_CHAT_Invitation *invitat
 }
 
 
+enum GNUNET_GenericReturnValue
+GNUNET_CHAT_invitation_is_direct (const struct GNUNET_CHAT_Invitation *invitation)
+{
+  GNUNET_CHAT_VERSION_ASSERT();
+
+  if ((invitation->key.code.public_bit) ||
+      (invitation->key.code.group_bit) ||
+      (invitation->key.code.feed_bit))
+    return GNUNET_NO;
+  else
+    return GNUNET_YES;
+}
+
+
 const struct GNUNET_CHAT_DiscourseId*
 GNUNET_CHAT_discourse_get_id (const struct GNUNET_CHAT_Discourse *discourse)
 {
@@ -3044,15 +3183,15 @@ GNUNET_CHAT_discourse_close (struct GNUNET_CHAT_Discourse *discourse)
   struct GNUNET_MESSENGER_Message msg;
   memset(&msg, 0, sizeof(msg));
 
-  msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBE;
+  msg.header.kind = GNUNET_MESSENGER_KIND_SUBSCRIBTION;
 
   util_shorthash_from_discourse_id(
     &(discourse->id),
-    &(msg.body.subscribe.discourse)
+    &(msg.body.subscription.discourse)
   );
 
-  msg.body.subscribe.time = GNUNET_TIME_relative_hton(GNUNET_TIME_relative_get_zero_());
-  msg.body.subscribe.flags = GNUNET_MESSENGER_FLAG_SUBSCRIPTION_UNSUBSCRIBE;
+  msg.body.subscription.time = GNUNET_TIME_relative_hton(GNUNET_TIME_relative_get_zero_());
+  msg.body.subscription.flags = GNUNET_MESSENGER_FLAG_SUBSCRIPTION_UNSUBSCRIBE;
 
   GNUNET_MESSENGER_send_message(
     discourse->context->room,
