@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2021--2024 GNUnet e.V.
+   Copyright (C) 2021--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -31,6 +31,7 @@
 #include <gnunet/gnunet_reclaim_lib.h>
 #include <gnunet/gnunet_reclaim_service.h>
 #include <gnunet/gnunet_time_lib.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -193,6 +194,27 @@ it_handle_iterate_contacts (void *cls,
   return it->cb(it->cls, it->handle, contact);
 }
 
+enum GNUNET_GenericReturnValue
+it_handle_find_own_contact (GNUNET_UNUSED void *cls,
+                            struct GNUNET_CHAT_Handle *handle,
+                            struct GNUNET_CHAT_Contact *contact)
+{
+  GNUNET_assert((handle) && (contact));
+
+  if (GNUNET_YES != GNUNET_CHAT_contact_is_owned(contact))
+    return GNUNET_YES;
+
+  const char *contact_key = GNUNET_CHAT_contact_get_key(contact);
+  const char *handle_key = GNUNET_CHAT_get_key(handle);
+
+  if ((!contact_key) || (!handle_key) ||
+      (0 != strcmp(contact_key, handle_key)))
+    return GNUNET_YES;
+
+  handle->own_contact = contact;
+  return GNUNET_NO;
+}
+
 struct GNUNET_CHAT_HandleIterateGroups
 {
   struct GNUNET_CHAT_Handle *handle;
@@ -320,24 +342,23 @@ task_contact_destruction (void *cls)
 
   util_shorthash_from_member(contact->member, &shorthash);
 
-  GNUNET_CONTAINER_multishortmap_remove(
-    contact->handle->contacts, &shorthash, contact
+  contact_leave (contact, contact->context);
+
+  const uint32_t other_contexts = GNUNET_CONTAINER_multihashmap_size(
+    contact->joined
   );
 
-  const struct GNUNET_HashCode *key = GNUNET_MESSENGER_room_get_key(
-    contact->context->room
-  );
-
-  GNUNET_CONTAINER_multihashmap_remove(
-    contact->handle->contexts, key, contact->context
-  );
+  if (0 >= other_contexts)
+    GNUNET_CONTAINER_multishortmap_remove(
+      contact->handle->contacts, &shorthash, contact
+    );
 
   context_delete(contact->context, GNUNET_YES);
-  context_destroy(contact->context);
 
   contact->destruction = NULL;
 
-  contact_destroy(contact);
+  if (0 >= other_contexts)
+    contact_destroy(contact);
 }
 
 void
