@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2022--2024 GNUnet e.V.
+   Copyright (C) 2022--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -26,6 +26,7 @@
 
 #include "gnunet_chat_handle.h"
 #include "gnunet_chat_lobby_intern.c"
+#include <gnunet/gnunet_messenger_service.h>
 
 struct GNUNET_CHAT_Lobby*
 lobby_create (struct GNUNET_CHAT_Handle *handle)
@@ -95,8 +96,14 @@ lobby_open (struct GNUNET_CHAT_Lobby *lobby,
     goto open_zone;
   }
 
-  struct GNUNET_HashCode key;
-  GNUNET_CRYPTO_random_block(GNUNET_CRYPTO_QUALITY_WEAK, &key, sizeof(key));
+  union GNUNET_MESSENGER_RoomKey key;
+  GNUNET_MESSENGER_create_room_key(
+    &key,
+    NULL,
+    GNUNET_NO,
+    GNUNET_NO,
+    GNUNET_NO
+  );
 
   struct GNUNET_MESSENGER_Room *room = GNUNET_MESSENGER_open_room(
     lobby->handle->messenger,
@@ -108,10 +115,8 @@ lobby_open (struct GNUNET_CHAT_Lobby *lobby,
 
   lobby->context = context_create_from_room(lobby->handle, room);
 
-  handle_send_room_name(lobby->handle, room);
-
   if (GNUNET_OK != GNUNET_CONTAINER_multihashmap_put(
-      lobby->handle->contexts, &key, lobby->context,
+      lobby->handle->contexts, &(key.hash), lobby->context,
       GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
   {
     context_destroy(lobby->context);
@@ -122,7 +127,7 @@ lobby_open (struct GNUNET_CHAT_Lobby *lobby,
   }
 
 open_zone:
-  util_lobby_name(&key, &name);
+  util_lobby_name(&(key.hash), &name);
 
   lobby->op = GNUNET_IDENTITY_create(
     lobby->handle->identity,
