@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2024 GNUnet e.V.
+   Copyright (C) 2024--2025 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -30,6 +30,13 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <string.h>
 
+enum GNUNET_MESSENGER_LinkType
+{
+  GNUNET_MESSENGER_LINK_DEFAULT = 0,
+  GNUNET_MESSENGER_LINK_DOTTED = 1,
+  GNUNET_MESSENGER_LINK_COMPOSITION = 2,
+};
+
 struct GNUNET_MESSENGER_Link
 {
   struct GNUNET_MESSENGER_Link *prev;
@@ -38,7 +45,7 @@ struct GNUNET_MESSENGER_Link
   struct GNUNET_HashCode hash;
   struct GNUNET_HashCode previous;
 
-  bool dotted;
+  enum GNUNET_MESSENGER_LinkType type;
 };
 
 struct GNUNET_MESSENGER_Tool
@@ -55,7 +62,10 @@ struct GNUNET_MESSENGER_Tool
 
   char *ego_name;
   char *room_name;
+  int public_room;
   int ignore_targets;
+  int ignore_epochs;
+  int simplify_merges;
 
   bool quit;
 };
@@ -81,11 +91,20 @@ idle (void *cls)
     const struct GNUNET_HashCode *hash = &(link->hash);
     const struct GNUNET_HashCode *previous = &(link->previous);
 
-    printf(
-      "X%s %s> ",
-      GNUNET_h2s(hash),
-      link->dotted? ".." : "--"
-    );
+    printf("X%s ", GNUNET_h2s(hash));
+
+    switch (link->type)
+    {
+      case GNUNET_MESSENGER_LINK_DOTTED:
+        printf("..> ");
+        break;
+      case GNUNET_MESSENGER_LINK_COMPOSITION:
+        printf("*-- ");
+        break;
+      default:
+        printf("--> ");
+        break;
+    }
     
     printf(
       "X%s\n",
@@ -101,7 +120,7 @@ static void
 add_link (struct GNUNET_MESSENGER_Tool *tool,
           const struct GNUNET_HashCode *hash,
           const struct GNUNET_HashCode *previous,
-          bool dotted)
+          enum GNUNET_MESSENGER_LinkType type)
 {
   size_t i;
   for (i = 0; i < sizeof(*previous); i++)
@@ -118,7 +137,7 @@ add_link (struct GNUNET_MESSENGER_Tool *tool,
   GNUNET_memcpy(&(link->hash), hash, sizeof(*hash));
   GNUNET_memcpy(&(link->previous), previous, sizeof(*previous));
 
-  link->dotted = dotted;
+  link->type = type;
 
   GNUNET_CONTAINER_DLL_insert(tool->head, tool->tail, link);
 }
@@ -143,6 +162,13 @@ message_callback (void *cls,
     tool->task = NULL;
   }
 
+  if ((tool->simplify_merges) &&
+      (GNUNET_MESSENGER_KIND_MERGE == message->header.kind))
+  {
+    printf("<> X%s\n", GNUNET_h2s(hash));
+    goto print_links;
+  }
+
   printf(
     "json X%s {",
     GNUNET_h2s(hash)
@@ -246,15 +272,51 @@ message_callback (void *cls,
         message->body.tag.tag
       );
       break;
+    case GNUNET_MESSENGER_KIND_ANNOUNCEMENT:
+      printf(
+        ",\n  \"identifier\":\"%s\"",
+        GNUNET_sh2s(&(message->body.announcement.identifier.hash))
+      );
+
+      printf(
+        ",\n  \"group\":\"%s\"",
+        (message->body.announcement.identifier.code.group_bit? "Y" : "N")
+      );
+      break;
+    case GNUNET_MESSENGER_KIND_SECRET:
+      printf(
+        ",\n  \"identifier\":\"%s\"",
+        GNUNET_sh2s(&(message->body.secret.identifier.hash))
+      );
+      break;
+    case GNUNET_MESSENGER_KIND_REVOLUTION:
+      printf(
+        ",\n  \"identifier\":\"%s\"",
+        GNUNET_sh2s(&(message->body.revolution.identifier.hash))
+      );
+      break;
+    case GNUNET_MESSENGER_KIND_GROUP:
+      printf(
+        ",\n  \"identifier\":\"%s\"",
+        GNUNET_sh2s(&(message->body.group.identifier.hash))
+      );
+      break;
+	case GNUNET_MESSENGER_KIND_AUTHORIZATION:
+      printf(
+        ",\n  \"identifier\":\"%s\"",
+        GNUNET_sh2s(&(message->body.authorization.identifier.hash))
+      );
+	  break;
     default:
       break;
   }
 
   printf("\n}\n");
 
+print_links:
   if (GNUNET_MESSENGER_KIND_MERGE == message->header.kind)
   {
-    add_link(tool, hash, &(message->body.merge.previous), false);
+    add_link(tool, hash, &(message->body.merge.previous), GNUNET_MESSENGER_LINK_DEFAULT);
 
     GNUNET_MESSENGER_get_message(
       room,
@@ -265,16 +327,47 @@ message_callback (void *cls,
   if (0 == tool->ignore_targets)
   {
     if (GNUNET_MESSENGER_KIND_REQUEST == message->header.kind)
-      add_link(tool, hash, &(message->body.request.hash), true);
+      add_link(tool, hash, &(message->body.request.hash), GNUNET_MESSENGER_LINK_DOTTED);
 
-    if (GNUNET_MESSENGER_KIND_DELETE == message->header.kind)
-      add_link(tool, hash, &(message->body.deletion.hash), true);
+    if (GNUNET_MESSENGER_KIND_DELETION == message->header.kind)
+      add_link(tool, hash, &(message->body.deletion.hash), GNUNET_MESSENGER_LINK_DOTTED);
 
     if (GNUNET_MESSENGER_KIND_TAG == message->header.kind)
-      add_link(tool, hash, &(message->body.tag.hash), true);
+      add_link(tool, hash, &(message->body.tag.hash), GNUNET_MESSENGER_LINK_DOTTED);
+
+    if (GNUNET_MESSENGER_KIND_APPEAL == message->header.kind)
+      add_link(tool, hash, &(message->body.appeal.event), GNUNET_MESSENGER_LINK_DOTTED);
+
+    if (GNUNET_MESSENGER_KIND_ACCESS == message->header.kind)
+      add_link(tool, hash, &(message->body.access.event), GNUNET_MESSENGER_LINK_DOTTED);
+
+    if (GNUNET_MESSENGER_KIND_GROUP == message->header.kind)
+    {
+      add_link(tool, hash, &(message->body.group.initiator), GNUNET_MESSENGER_LINK_DOTTED);
+      add_link(tool, hash, &(message->body.group.partner), GNUNET_MESSENGER_LINK_DOTTED);
+    }
+
+    if (GNUNET_MESSENGER_KIND_AUTHORIZATION == message->header.kind)
+      add_link(tool, hash, &(message->body.authorization.event), GNUNET_MESSENGER_LINK_DOTTED);
   }
 
-  add_link(tool, hash, &(message->header.previous), false);
+  if (0 == tool->ignore_epochs)
+  {
+    if (GNUNET_MESSENGER_KIND_JOIN == message->header.kind)
+      add_link(tool, hash, &(message->body.join.epoch), GNUNET_MESSENGER_LINK_COMPOSITION);
+
+    if (GNUNET_MESSENGER_KIND_LEAVE == message->header.kind)
+      add_link(tool, hash, &(message->body.leave.epoch), GNUNET_MESSENGER_LINK_COMPOSITION);
+
+    if ((GNUNET_MESSENGER_KIND_MERGE == message->header.kind) &&
+        (0 != GNUNET_memcmp(&(message->body.merge.epochs[0]), &(message->body.merge.epochs[1]))))
+    {
+      add_link(tool, hash, &(message->body.merge.epochs[0]), GNUNET_MESSENGER_LINK_COMPOSITION);
+      add_link(tool, hash, &(message->body.merge.epochs[1]), GNUNET_MESSENGER_LINK_COMPOSITION);
+    }
+  }
+
+  add_link(tool, hash, &(message->header.previous), GNUNET_MESSENGER_LINK_DEFAULT);
 
   GNUNET_MESSENGER_get_message(
     room,
@@ -332,11 +425,20 @@ ego_lookup (void *cls,
     );
   else
     memset(&hash, 0, sizeof(hash));
+
+  union GNUNET_MESSENGER_RoomKey rkey;
+  GNUNET_MESSENGER_create_room_key(
+    &rkey,
+    tool->room_name,
+    tool->public_room? GNUNET_YES : GNUNET_NO,
+    GNUNET_YES,
+    GNUNET_NO
+  );
   
   GNUNET_MESSENGER_enter_room(
     tool->handle,
     &peer,
-    &hash
+    &rkey
   );
 }
 
@@ -389,12 +491,30 @@ main (int argc,
       "name of room to read messages from",
       &(tool.room_name)
     ),
+    GNUNET_GETOPT_option_flag(
+      'P',
+      "public",
+      "disable forward secrecy in public rooms",
+      &(tool.public_room)
+    ),
     GNUNET_GETOPT_option_flag(
       'i',
       "ignore-targets",
       "ignore indirect connections between messages and their targets",
       &(tool.ignore_targets)
     ),
+    GNUNET_GETOPT_option_flag(
+      'e',
+      "ignore-epochs",
+      "ignore indirect connections between epoch messages and their previous epoch",
+      &(tool.ignore_epochs)
+    ),
+    GNUNET_GETOPT_option_flag(
+      'm',
+      "simplify-merges",
+      "simplify merge messages in the message graph",
+      &(tool.simplify_merges)
+    ),
     GNUNET_GETOPT_OPTION_END
   };
 
